function OnInit(initData)
{
	initData.name = "ses";
	initData.vars("name") = initData.name;
	initData.desc = "Front end to everything (se) and command line (es)";
	initData.copyright = "(c) 2018 aussieboykie";
	initData.version = "2.1 (March 26 2018)";
	initData.vars("version") = initData.version;
	initData.vars("minver") = "12.5"; // Added in v2.1
	initData.default_enable = true;
	
	// This script adds four commands:
	// se = Front end to everything.exe
	// es = Front end to es.exe
	// se_p = Front end to everything.exe with validation prompt
	// es_p = Front end to es.exe with validation prompt
	
	var n, cmd, cmds = DOpus.create.vector("se", "es", "se_p", "es_p");
	for (var i = 0; i < cmds.length; i++){
		cmd = initData.AddCommand();
		cmd.name = cmds(i);
		n = i + 1;
		cmd.method = "do_cmd" + n;
		cmd.desc = initData.desc;
		cmd.label = initData.name;
		cmd.template = "args/r";
	}
	
	var pgms = String(DOpus.fsutil.resolve("/programfiles"));
	var home = String(DOpus.fsutil.resolve("/homeroot"));
	var xusb = (pgms.slice(0,2)!=home.slice(0,2));
	// xusb will be true if this script is run from a USB device
	// Not sure if this test is foolproof or if there is a better way
	initData.vars("xusb") = xusb;
	
	initData.config_desc = DOpus.create.map();
	initData.config.dbg = false;
	initData.config_desc("dbg") = 'Set to TRUE to write trace information to "Other" log.';
	initData.config.espane = "NEWTAB=findexisting USEQUALKEYS";
	initData.config_desc("espane") = "Specify how collection is shown by es or es_p";
	initData.config.defterms = "dm:last5mins !path:/temp !path:/windows";
	initData.config_desc("defterms") = "Default search criteria";
	initData.config.espath = (xusb) ? "/homeroot\\utilities\\everything" : "";
	initData.config_desc("espath") = "Path to es.exe";
	initData.config.evpath = (xusb) ? "/homeroot\\utilities\\everything" : "";
	initData.config_desc("evpath") = "Path to everything.exe";
	initData.config.showemptycoll = false;
	initData.config_desc("showemptycoll") = "Set to TRUE to show empty collection.";
}

//-- Global Variables

var d, f, c, fsu;
var src, tgt, dlg;
var msg, btns, title;
var item, pgm, str;
var i, n;
var dbg; 
var defterms, espath, evpath, showemptycoll, minver; 
var xusb;
var argstring, quot = '"';
var ForReading = 1, ForWriting = 2, ForAppending = 8;

function abort(msg)
{
	title = "Unable to continue";
	btns = "OK";
	dlg.request(msg, btns, title);
}

function build_argstring()
{
	if (dbg) DOpus.output("got_arg.args = " + f.args.got_arg.args);
	argstring = (f.args.got_arg.args) ? f.args.args : "";
	if (dbg) DOpus.output("raw args = " + argstring);
	argstring = argstring.trim();
	// May have been called with arguments - e.g. args wfn:*2018*.jpg
	if (argstring!=""){
		// Deal with mistaken use of args = something
		if (argstring.indexOf("=")==0) argstring = argstring.slice(1).trim();
	}
	// If no args were supplied define default search terms
	else if (src.selected.count==1) argstring = "wfn:" + quot + src.selected(0).name + quot;
	// Apply user supplied defaults if no arguments were supplied
	return (argstring=="") ? Script.config.defterms : argstring;
}

function do_cmd1(clickData, validate)
{
	if (init_common(clickData)==false) {abort(msg); return}; // Minver test added in v2.1
	
	// Attempt to locate everything.exe
	pgm = "everything.exe";
	var ev = locate_pgm(evpath, pgm);
	if (dbg) DOpus.output("ev = " + ev);
	if (ev=="") {abort(pgm + " not found"); return ""};
	// Establish search criteria
	str = build_argstring();
	if ((str=="") || (validate==true)) str = validate_search_terms(str, pgm);
	if (str=="") return;
	str = resolve_aliases(str);
	str = str.replace(/"/g, quot+quot+quot);
	str = quot + str + quot;
	// Execute everything.exe
	argstring = '"' + ev + '" -s ' + str;
	if (dbg) DOpus.output("argstring = " + argstring);
	c.runcommand(argstring);
}

function do_cmd2(clickData, validate)
{
	if (init_common(clickData)==false) {abort(msg); return}; // Minvar test added in v2.1
	
	// Attempt to locate es.exe
	pgm = "es.exe";
	var es = locate_pgm(espath, pgm);
	if (dbg) DOpus.output("es = " + es);
	if (es=="") {abort(pgm + " not found"); return ""};
	var es_path = es.slice(0, es.length - 6);
	if (dbg) DOpus.output("es_path = " + es_path);
	// Everything needs to be running
	if (DOpus.Create.Sysinfo.FindProcess("everything.exe")==0){
		var ev = locate("everything.exe", Script.config.evpath);
		if (dbg) DOpus.output("ev = " + ev);
		if (ev!="") c.runcommand(ev);
		else{
			abort("everything.exe not found");
			return "";
		}
	}
	// Execute es.exe with user supplied or default arguments
	// Location of temp files is different for XP
	str = (DOpus.version.winver.xp==true) ? "/homeroot" : "/temp";
	var bat = fsu.resolve(str + "\\es.cmd");
	var tmp = fsu.resolve(str + "\\es.out");
	// Establish search criteria
	str = build_argstring();
	if ((str=="") || (validate==true)) str = validate_search_terms(str, pgm);
	if (str=="") return;
	str = resolve_aliases(str);
	var validated_args = str;
	if (dbg) DOpus.output("validated_args = " + validated_args);
	// Handle things that need to be escaped
	str = str.replace(/>/g, "gt:");
	str = str.replace(/</g, "lt:");
	str = str.replace(/\|/g, "^|");
	// Add es options
	str = str + " -dm -size -full-path-and-name -sort-date-modified -export-txt ";
	// Finalise es command
	argstring = "es.exe " + str + tmp;
	if (dbg) DOpus.output("argstring = " + argstring);
	// Generate a batch command to execute the search
	var fso = new ActiveXObject("Scripting.FileSystemObject");
	var f = fso.OpenTextFile(bat,ForWriting,true);
	f.WriteLine("pushd " + es_path);
	f.WriteLine(argstring);
	f.WriteLine("popd");
	f.close();
	// Run the batch command to create an output file containing matches
	var objWShell = new ActiveXObject("WScript.Shell");
	objWShell.Run(bat, 0, true);
	// Convert es output (if there is any) into an Opus collection and display it
	if ((showemptycoll) || (fsu.getitem(tmp).size>0)){
		argstring = "/home\\dopusrt.exe /col import /clear /create everything " + quot + tmp + quot;
		if (dbg) DOpus.output("argstring = " + argstring);
		c.runcommand(argstring);
		if (dbg) DOpus.output("espane = " + Script.config.espane);
		if (DOpus.listers.count!=0) str = Script.config.espane;
		else str = "NEW=notree,nodual,noutilitypanel,nometapane,noviewpane";
		if (str=="") str = "NEWTAB=findexisting USEQUALKEYS";
		argstring = "Go Coll://Everything " + str;
		if (dbg) DOpus.output("argstring = " + argstring);
		c.runcommand(argstring);
	}
	else abort("No match found for the following search term(s):\n\n" + validated_args);
}

function do_cmd3(clickData)
{
	do_cmd1(clickData, true)
}

function do_cmd4(clickData)
{
	do_cmd2(clickData, true)
}

function init_common(clickData)
{
	dbg = Script.config.dbg;
	defterms = Script.config.defterms;
	espath = Script.config.espath;
	evpath = Script.config.evpath;
	showemptycoll = Script.config.showemptycoll;
	xusb = Script.vars("xusb");
	if (dbg){
		DOpus.output("defterms = " + defterms);
		DOpus.output("espath = " + espath);
		DOpus.output("evpath = " + evpath);
		DOpus.output("showemptycoll = " + showemptycoll);
		DOpus.output("xusb = " + xusb);
		DOpus.output("home = " + String(DOpus.fsutil.resolve("/homeroot")));
		DOpus.output("pgms = " + String(DOpus.fsutil.resolve("/programfiles")));
	}
	d = clickData;
	f = d.func;
	c = f.command;
	fsu = DOpus.FSUtil;
	src = f.sourcetab;
	if (typeof f.sourcetab!="object"){
		dlg = DOpus.dlg;
		if (dbg) DOpus.output("typeof f.sourcetab = " + typeof f.sourcetab);
	}
	else dlg = src.dlg;
	// If it's not already present, add a trim() method to the JScript String prototype
	if(!String.prototype.trim) String.prototype.trim = function(){return this.replace(/^\s+|\s+$/g,'');};
	str = Script.vars("name")+" version "+Script.vars("version")+" starting";
	str = (Script.vars("xusb")) ? str + " from USB" : str;
	if (dbg) DOpus.output(str);
	minver = Script.vars("minver"); // Added in v2.1
	msg = "Directory Opus version must be at least " + minver; // Added in v2.1
	return DOpus.version.AtLeast(minver); // Added in v2.1
}

function locate_pgm(preferred_path, pgm)
{
	var spec;
	if (preferred_path!=""){
		spec = fsu.resolve(preferred_path + "\\" + pgm);
		if (fsu.exists(spec)) return String(spec);
	}
	spec = fsu.resolve("/programfiles\\everything\\" + pgm);
	if (fsu.exists(spec)) return String(spec); 
	spec = fsu.resolve("/programfilesx86\\everything\\" + pgm);
	if (fsu.exists(spec)) return String(spec);
	else return "";
}

function resolve_aliases(str)
{
	// Example: path:/dopusdata dm:last14days
	var l = str.indexOf("/");
	if (l<0) return str;
	var pfx, alias, sfx;
	pfx = (l==0) ? "" : str.slice(0,l);
	// pfx will be path:
	sfx = str.slice(l);
	// sfx will be /dopusdata dm:last14days
	// The alias may be followed by other stuff, as in this example
	l = sfx.slice(1).search(/[^a-zA-Z0-9]/);
	// In our example, l will be 9
	if (l<0){
		alias = sfx;
		sfx = "";
	}
	else{
		alias = sfx.slice(0,l+1);
		sfx = sfx.slice(l+1);
	}
	alias = String(fsu.resolve(alias));
	if (alias.indexOf(" ")>0) alias = quot + alias + quot;
	res = pfx + alias + sfx;
	if (dbg) DOpus.output("res = " + res);
	return (alias.slice(0,1)=="/") ? res : resolve_aliases(res);
}

function validate_search_terms(str, pgm)
{
	title = "Edit/Validate Search Terms";
	btns = "Anywhere|Here|Cancel";
	msg = pgm + " search terms:"
	var valstr = dlg.getstring(msg, str, 0, btns, title);
	if (dlg.result==0) return "";
	else if (dlg.result==2) return valstr + ' path:"' + src.path + '"';
	else return valstr;
}